==> iwrite-1.0.1.4.tar.gz: iwrite-1.0.1.4/src/item_page_number.c <==
/* 
vim:expandtab:softtabstop=2:tabstop=2:shiftwidth=2:nowrap:ruler
*/
/*
Copyright (c) 2015-2016, iwrite authors 
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
#include "item_page_number.h"

extern void
item_page_number_assign(
  struct item_page_number*const         o_page_number,
  struct item_method*const              o_method)
{

  memset(o_page_number, 0, sizeof(*o_page_number));

  (*o_page_number).m_len_x= 30.0;
  (*o_page_number).m_len_y= 18.0;
  (*o_page_number).m_font= pango_font_description_new();
  (*o_page_number).m_border_style= line_style_solid;
  (*o_page_number).m_border_thickness= 1.0;

  ITEM_METHOD_ASSIGN(o_method, page_number);

  return;
}

extern void
item_page_number_discharge(
  struct item_page_number*const         io_page_number)
{

  if ((*io_page_number).m_font)
  {
    pango_font_description_free((*io_page_number).m_font);
  }

  memset(io_page_number, 0, sizeof(*io_page_number));

  return;
}

static void
item_page_number_get_text(
  char                                  o_text[1+SPECIAL_FIELD_VALUE_MAX_LEN],
  struct item_page_number*const         i_page_number,
  struct item_draw_context const*const  i_ctx)
{
  char                                  l_format[1+FIELD_FORMAT_MAX_LEN];
  unsigned long                         l_page_number; 

  memset(o_text, 0, 1+SPECIAL_FIELD_VALUE_MAX_LEN);

  do
  {

    if (item_draw_design == (*i_ctx).m_option.m_type)
    {
      l_page_number= 1;
    }
    else
    {
      l_page_number= (*i_page_number).m_page_number;
    }

    if ((*i_page_number).m_format[0])
    {
      strcpy(l_format, (*i_page_number).m_format);
    }
    else
    {
      strcpy(l_format, "%6ld");
    }

    snprintf(
      o_text, 
      SPECIAL_FIELD_VALUE_MAX_LEN-1,
      l_format,
      l_page_number);

  }while(0);

  return;
}

extern void
item_page_number_draw(
  struct item_page_number*const         i_page_number,
  struct item_draw_context const*const  i_ctx)
{
  PangoFontDescription*                 l_font;
  PangoLayout*                          l_layout;
  char                                  l_text[1+SPECIAL_FIELD_VALUE_MAX_LEN];
  size_t                                l_modulus;

  l_font= 0;
  l_layout= 0;

  do
  {

    if (item_draw_print == (*i_ctx).m_option.m_type)
    {
      l_modulus= ((*i_page_number).m_page_number % 2);

      if (page_number_all != (*i_page_number).m_type)
      {
        if (page_number_even == (*i_page_number).m_type)
        {
          if (l_modulus)
          {
            break;
          }
        }
        else if (page_number_odd == (*i_page_number).m_type)
        {
          if (0 == l_modulus)
          {
            break;
          }
        }
      }
    }

    cairo_save((*i_ctx).m_cr);

    cairo_rectangle(
      (*i_ctx).m_cr,
      (*i_page_number).m_pos_x,
      (*i_page_number).m_pos_y,
      (*i_page_number).m_len_x,
      (*i_page_number).m_len_x);

    cairo_clip((*i_ctx).m_cr);

    cairo_set_source_rgb(
      (*i_ctx).m_cr,
      (*i_page_number).m_color.red,
      (*i_page_number).m_color.green,
      (*i_page_number).m_color.blue);

    l_layout= pango_cairo_create_layout((*i_ctx).m_cr);
    pango_layout_set_font_description(l_layout, (*i_page_number).m_font); 
    cairo_translate((*i_ctx).m_cr, (*i_page_number).m_pos_x, (*i_page_number).m_pos_y);

    item_page_number_get_text(l_text, i_page_number, i_ctx);

    pango_layout_set_text(l_layout, l_text, -1);

    pango_cairo_update_layout((*i_ctx).m_cr, l_layout);
    pango_cairo_show_layout((*i_ctx).m_cr, l_layout);

    cairo_restore((*i_ctx).m_cr);

    if ((*i_page_number).m_border)
    {

      item_common_cairo_dash_set((*i_ctx).m_cr, (*i_page_number).m_border_style);
      cairo_set_line_width((*i_ctx).m_cr, (*i_page_number).m_border_thickness);
      cairo_set_source_rgb(
        (*i_ctx).m_cr,
        (*i_page_number).m_border_color.red,
        (*i_page_number).m_border_color.green,
        (*i_page_number).m_border_color.blue);

      cairo_rectangle(
        (*i_ctx).m_cr,
        (*i_page_number).m_pos_x,
        (*i_page_number).m_pos_y,
        (*i_page_number).m_len_x,
        (*i_page_number).m_len_y);

      cairo_stroke((*i_ctx).m_cr);
    }

    if (0 == (*i_ctx).m_selected)
    {
      if ((*i_ctx).m_option.m_highlight &&
        (item_draw_design == (*i_ctx).m_option.m_type))
      {
        item_common_box_draw_hightlight(
          (*i_ctx).m_cr, 
          (struct geom_rect*)i_page_number,
          (*i_page_number).m_border_thickness);
      }
      break;
    }

    item_common_box_draw_grips((*i_ctx).m_cr, (struct geom_rect*)i_page_number);

  }while(0);

  if (l_font)
  {
    pango_font_description_free(l_font);
  }

  if (l_layout)
  {
    g_object_unref(l_layout);
  }

  return;
}

extern void
item_page_number_get_bbox(
  struct geom_rect*const                o_bbox,
  int*const                             o_overlay,
  void const*const                      i_object)
{
  (*o_bbox)= *(struct geom_rect*)i_object;
  (*o_overlay)= 0;
  return;
}

extern void
item_page_number_in_event(
  enum event_type*const                 o_type,
  struct item_page_number const*const   i_page_number,
  struct item_in_event const*const      i_ctx)
{

  item_common_box_in_event(o_type, i_ctx, (struct geom_rect*)i_page_number);

  return;
}

extern void
item_page_number_move(
  struct item_page_number*const         io_page_number,
  struct geom_point const*const         i_event)
{
  item_common_box_move((struct geom_rect*)io_page_number, i_event);
  return;
}

extern void
item_page_number_normalize(
  struct item_page_number*const         io_page_number,
  double const                          i_bound_width,
  double const                          i_bound_height)
{
  item_common_box_normalize(
    (struct geom_rect*)io_page_number, 
    i_bound_width,
    i_bound_height);
  return;
}

extern int
item_page_number_read(
  GError**                              o_error,
  struct item_page_number*const         io_page_number,
  struct bson_node *const               i_element)
{
  struct bson_node *                    l_element;
  int                                   l_found;
  enum element_tag_type                 l_type;

  l_element= i_element;

  do
  {

    if (0 == l_element)
    {
      break;
    }

    if (bson_type_end == (*l_element).m_type)
    {
      break;
    }

    item_common_lookup(&l_found, &l_type, (*l_element).m_ename);

    if (l_found)
    {
        switch(l_type)
        {
          case element_tag_border:
            if (bson_type_int32 == (*l_element).m_type)
            {
              (*io_page_number).m_border= (*l_element).m_object.m_int32;
            }
            break;
          case element_tag_border_color_red:
            if (bson_type_float == (*l_element).m_type)
            {
              (*io_page_number).m_border_color.red= (*l_element).m_object.m_float;
            }
            break;
          case element_tag_border_color_green:
            if (bson_type_float == (*l_element).m_type)
            {
              (*io_page_number).m_border_color.green= (*l_element).m_object.m_float;
            }
            break;
          case element_tag_border_color_blue:
            if (bson_type_float == (*l_element).m_type)
            {
              (*io_page_number).m_border_color.blue= (*l_element).m_object.m_float;
            }
            break;
          case element_tag_border_thickness:
            if (bson_type_float == (*l_element).m_type)
            {
              (*io_page_number).m_border_thickness= (*l_element).m_object.m_float;
            }
            break;
          case element_tag_color_red:
            if (bson_type_float == (*l_element).m_type)
            {
              (*io_page_number).m_color.red= (*l_element).m_object.m_float;
            }
            break;
          case element_tag_color_green:
            if (bson_type_float == (*l_element).m_type)
            {
              (*io_page_number).m_color.green= (*l_element).m_object.m_float;
            }
            break;
          case element_tag_color_blue:
            if (bson_type_float == (*l_element).m_type)
            {
              (*io_page_number).m_color.blue= (*l_element).m_object.m_float;
            }
            break;
          case element_tag_fill:
            if (bson_type_int32 == (*l_element).m_type)
            {
              (*io_page_number).m_fill= (*l_element).m_object.m_int32;
            }
            break;
          case element_tag_fill_color_red:
            if (bson_type_float == (*l_element).m_type)
            {
              (*io_page_number).m_fill_color.red= (*l_element).m_object.m_float;
            }
            break;
          case element_tag_fill_color_green:
            if (bson_type_float == (*l_element).m_type)
            {
              (*io_page_number).m_fill_color.green= (*l_element).m_object.m_float;
            }
            break;
          case element_tag_fill_color_blue:
            if (bson_type_float == (*l_element).m_type)
            {
              (*io_page_number).m_fill_color.blue= (*l_element).m_object.m_float;
            }
            break;
          case element_tag_font:
            if (bson_type_string == (*l_element).m_type)
            {
              (*io_page_number).m_font= pango_font_description_from_string(
                (*l_element).m_object.m_string.m_text);
            }
            break;
          case element_tag_len_x:
            if (bson_type_float == (*l_element).m_type)
            {
              (*io_page_number).m_len_x= (*l_element).m_object.m_float;
            }
            break;
          case element_tag_len_y:
            if (bson_type_float == (*l_element).m_type)
            {
              (*io_page_number).m_len_y= (*l_element).m_object.m_float;
            }
            break;
          case element_tag_pos_x:
            if (bson_type_float == (*l_element).m_type)
            {
              (*io_page_number).m_pos_x= (*l_element).m_object.m_float;
            }
            break;
          case element_tag_pos_y:
            if (bson_type_float == (*l_element).m_type)
            {
              (*io_page_number).m_pos_y= (*l_element).m_object.m_float;
            }
            break;
          case element_tag_border_style:
            if (bson_type_int32 == (*l_element).m_type)
            {
              (*io_page_number).m_border_style= (*l_element).m_object.m_int32;
            }
            break;
          case element_tag_format:
            if (bson_type_string == (*l_element).m_type)
            {
              strncpy(
                (char*)&(*io_page_number).m_format,
                (*l_element).m_object.m_string.m_text,
                sizeof((*io_page_number).m_format));
            }
            break;
          case element_tag_page_number:
            if (bson_type_int32 == (*l_element).m_type)
            {
              (*io_page_number).m_page_number= (*l_element).m_object.m_int32;
            }
            break;
          case element_tag_type:
            if (bson_type_int32 == (*l_element).m_type)
            {
              (*io_page_number).m_type= (*l_element).m_object.m_int32;
            }
            break;
          default:
            break;
        }
    }

    l_element= (*l_element).m_next;

  }while(1);

  return 0;
}

extern void
item_page_number_resize(
  struct item_page_number*const         io_page_number,
  struct item_resize_event const*const  i_ctx)
{

  item_common_box_resize((struct geom_rect *)io_page_number, i_ctx);

  return;
}

extern int
item_page_number_write(
  GError**                              o_error,
  FILE*const                            io_fp,
  struct item_page_number const*const   i_page_number)
{
  int                                   l_exit;
  int32_t                               l_int32;
  char                                  l_octet;
  int32_t                               l_offset;
  int32_t                               l_size;
  char*                                 l_text;

  l_exit= 0;
  l_text= 0;

  /* array */
  l_octet= bson_type_array;
  fwrite(&l_octet, 1, 1, io_fp); 
  fprintf(io_fp, "page_number%c",0);

  /* document */
  l_offset= ftell(io_fp);
  l_int32= 0;
  fwrite(&l_int32, sizeof(int32_t), 1, io_fp); 

  /* pos_x */
  l_octet= bson_type_float;
  fwrite(&l_octet, 1, 1, io_fp); 
  fprintf(io_fp, "pos_x%c",0);
  fwrite(&(*i_page_number).m_pos_x, sizeof(double), 1, io_fp);

  /* pos_y */
  l_octet= bson_type_float;
  fwrite(&l_octet, 1, 1, io_fp); 
  fprintf(io_fp, "pos_y%c",0);
  fwrite(&(*i_page_number).m_pos_y, sizeof(double), 1, io_fp);

  /* len_x */
  l_octet= bson_type_float;
  fwrite(&l_octet, 1, 1, io_fp); 
  fprintf(io_fp, "len_x%c",0);
  fwrite(&(*i_page_number).m_len_x, sizeof(double), 1, io_fp);

  /* len_y */
  l_octet= bson_type_float;
  fwrite(&l_octet, 1, 1, io_fp); 
  fprintf(io_fp, "len_y%c",0);
  fwrite(&(*i_page_number).m_len_y, sizeof(double), 1, io_fp);

  /* font */
  l_text= pango_font_description_to_string((*i_page_number).m_font);
  if (l_text[0])
  {
    l_octet= bson_type_string;
    fwrite(&l_octet, 1, 1, io_fp); 
    fprintf(io_fp, "font%c",0);
    l_int32= (1+strlen(l_text));
    fwrite(&l_int32, sizeof(l_int32), 1, io_fp); 
    fwrite(l_text, l_int32, 1, io_fp);
  }
  g_free(l_text);
  l_text= 0;

  /* color - red */
  l_octet= bson_type_float;
  fwrite(&l_octet, 1, 1, io_fp); 
  fprintf(io_fp, "color_red%c",0);
  fwrite(&(*i_page_number).m_color.red, sizeof(double), 1, io_fp);

  /* color - green */
  l_octet= bson_type_float;
  fwrite(&l_octet, 1, 1, io_fp); 
  fprintf(io_fp, "color_green%c",0);
  fwrite(&(*i_page_number).m_color.green, sizeof(double), 1, io_fp);

  /* color - blue */
  l_octet= bson_type_float;
  fwrite(&l_octet, 1, 1, io_fp); 
  fprintf(io_fp, "color_blue%c",0);
  fwrite(&(*i_page_number).m_color.blue, sizeof(double), 1, io_fp);

  /* border */
  l_octet= bson_type_int32;
  fwrite(&l_octet, 1, 1, io_fp); 
  fprintf(io_fp, "border%c",0);
  fwrite(&(*i_page_number).m_border, sizeof(int32_t), 1, io_fp);

  /* border_style */
  l_octet= bson_type_int32;
  fwrite(&l_octet, 1, 1, io_fp); 
  fprintf(io_fp, "border_style%c",0);
  fwrite(&(*i_page_number).m_border_style, sizeof(int32_t), 1, io_fp);

  /* border_thickness */
  l_octet= bson_type_float;
  fwrite(&l_octet, 1, 1, io_fp); 
  fprintf(io_fp, "border_thickness%c",0);
  fwrite(&(*i_page_number).m_border_thickness, sizeof(double), 1, io_fp);

  /* border_color - red */
  l_octet= bson_type_float;
  fwrite(&l_octet, 1, 1, io_fp); 
  fprintf(io_fp, "border_color_red%c",0);
  fwrite(&(*i_page_number).m_border_color.red, sizeof(double), 1, io_fp);

  /* border_color - green */
  l_octet= bson_type_float;
  fwrite(&l_octet, 1, 1, io_fp); 
  fprintf(io_fp, "border_color_green%c",0);
  fwrite(&(*i_page_number).m_border_color.green, sizeof(double), 1, io_fp);

  /* border_color - blue */
  l_octet= bson_type_float;
  fwrite(&l_octet, 1, 1, io_fp); 
  fprintf(io_fp, "border_color_blue%c",0);
  fwrite(&(*i_page_number).m_border_color.blue, sizeof(double), 1, io_fp);

  /* fill */
  l_octet= bson_type_int32;
  fwrite(&l_octet, 1, 1, io_fp); 
  fprintf(io_fp, "fill%c",0);
  fwrite(&(*i_page_number).m_fill, sizeof(int32_t), 1, io_fp);

  /* fill color - red */
  l_octet= bson_type_float;
  fwrite(&l_octet, 1, 1, io_fp); 
  fprintf(io_fp, "fill_red%c",0);
  fwrite(&(*i_page_number).m_fill_color.red, sizeof(double), 1, io_fp);

  /* fill color - green */
  l_octet= bson_type_float;
  fwrite(&l_octet, 1, 1, io_fp); 
  fprintf(io_fp, "fill_green%c",0);
  fwrite(&(*i_page_number).m_fill_color.green, sizeof(double), 1, io_fp);

  /* fill color - blue */
  l_octet= bson_type_float;
  fwrite(&l_octet, 1, 1, io_fp); 
  fprintf(io_fp, "fill_blue%c",0);
  fwrite(&(*i_page_number).m_fill_color.blue, sizeof(double), 1, io_fp);

  /* page_number */
  l_octet= bson_type_int32;
  fwrite(&l_octet, 1, 1, io_fp); 
  fprintf(io_fp, "page_number%c",0);
  fwrite(&(*i_page_number).m_page_number, sizeof(int32_t), 1, io_fp);

  /* type */
  l_octet= bson_type_int32;
  fwrite(&l_octet, 1, 1, io_fp); 
  fprintf(io_fp, "type%c",0);
  fwrite(&(*i_page_number).m_type, sizeof(int32_t), 1, io_fp);

  /* format */
  l_octet= bson_type_string;
  fwrite(&l_octet, 1, 1, io_fp); 
  fprintf(io_fp, "format%c",0);
  l_int32= sizeof((*i_page_number).m_format);
  fwrite(&l_int32, sizeof(int32_t), 1, io_fp); 
  fwrite(&(*i_page_number).m_format, l_int32, 1, io_fp);

  /* end of document */
  l_octet= 0;
  fwrite(&l_octet, 1, 1, io_fp); 

  /* document */
  l_int32= ftell(io_fp);
  l_size= (l_int32 - l_offset);
  fseek(io_fp, l_offset, SEEK_SET);
  fwrite(&l_size, sizeof(l_size), 1, io_fp); 


  fseek(io_fp, 0, SEEK_END);

  return l_exit;
}

==> iwrite-2.0.1.0.tar.gz: iwrite-2.0.1.0/src/item_page_number.c <==
/* 
vim:expandtab:softtabstop=2:tabstop=2:shiftwidth=2:nowrap:ruler
*/
/*
Copyright (c) 2015-2016, iwrite authors 
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
#include "item_page_number.h"

extern void
item_page_number_assign(
  struct item_page_number*const         o_page_number,
  struct item_method*const              o_method)
{

  memset(o_page_number, 0, sizeof(*o_page_number));

  (*o_page_number).m_len_x= 30.0;
  (*o_page_number).m_len_y= 18.0;
  (*o_page_number).m_font= pango_font_description_new();
  (*o_page_number).m_color.alpha= 1.0;
  (*o_page_number).m_border.m_style= line_style_solid;
  (*o_page_number).m_border.m_thickness= 1.0;
  (*o_page_number).m_border.m_color.alpha= 1.0;
  (*o_page_number).m_fill.m_color.alpha= 1.0;

  ITEM_METHOD_ASSIGN(o_method, page_number);

  return;
}

extern void
item_page_number_discharge(
  struct item_page_number*const         io_page_number)
{

  if ((*io_page_number).m_font)
  {
    pango_font_description_free((*io_page_number).m_font);
  }

  memset(io_page_number, 0, sizeof(*io_page_number));

  return;
}

static void
item_page_number_get_text(
  char                                  o_text[1+SPECIAL_FIELD_VALUE_MAX_LEN],
  struct item_page_number*const         i_page_number,
  struct item_draw_context const*const  i_ctx)
{
  char                                  l_format[1+FIELD_FORMAT_MAX_LEN];
  unsigned long                         l_page_number; 

  memset(o_text, 0, 1+SPECIAL_FIELD_VALUE_MAX_LEN);

  do
  {

    if (item_draw_design == (*i_ctx).m_option.m_type)
    {
      l_page_number= 1;
    }
    else
    {
      l_page_number= (*i_page_number).m_page_number;
    }

    if ((*i_page_number).m_format[0])
    {
      g_strlcpy(l_format, (*i_page_number).m_format, sizeof(l_format));
    }
    else
    {
      g_strlcpy(l_format, "%6ld", sizeof(l_format));
    }

    snprintf(
      o_text, 
      SPECIAL_FIELD_VALUE_MAX_LEN-1,
      l_format,
      l_page_number);

  }while(0);

  return;
}

extern void
item_page_number_draw(
  struct item_page_number*const         i_page_number,
  struct item_draw_context const*const  i_ctx)
{
  PangoFontDescription*                 l_font;
  PangoLayout*                          l_layout;
  char                                  l_text[1+SPECIAL_FIELD_VALUE_MAX_LEN];
  size_t                                l_modulus;

  l_font= 0;
  l_layout= 0;

  do
  {

    if (item_draw_print == (*i_ctx).m_option.m_type)
    {
      l_modulus= ((*i_page_number).m_page_number % 2);

      if (page_number_all != (*i_page_number).m_type)
      {
        if (page_number_even == (*i_page_number).m_type)
        {
          if (l_modulus)
          {
            break;
          }
        }
        else if (page_number_odd == (*i_page_number).m_type)
        {
          if (0 == l_modulus)
          {
            break;
          }
        }
      }
    }

    if ((*i_page_number).m_fill.m_active)
    {
      cairo_set_source_rgba(
        (*i_ctx).m_cr,
        (*i_page_number).m_fill.m_color.red,
        (*i_page_number).m_fill.m_color.green,
        (*i_page_number).m_fill.m_color.blue,
        (*i_page_number).m_fill.m_color.alpha);

      cairo_rectangle(
        (*i_ctx).m_cr,
        (*i_page_number).m_pos_x,
        (*i_page_number).m_pos_y,
        (*i_page_number).m_len_x,
        (*i_page_number).m_len_y);

      cairo_fill((*i_ctx).m_cr);
    }

    cairo_save((*i_ctx).m_cr);

    cairo_rectangle(
      (*i_ctx).m_cr,
      (*i_page_number).m_pos_x,
      (*i_page_number).m_pos_y,
      (*i_page_number).m_len_x,
      (*i_page_number).m_len_x);

    cairo_clip((*i_ctx).m_cr);

    cairo_set_source_rgba(
      (*i_ctx).m_cr,
      (*i_page_number).m_color.red,
      (*i_page_number).m_color.green,
      (*i_page_number).m_color.blue,
      (*i_page_number).m_color.alpha);

    l_layout= pango_cairo_create_layout((*i_ctx).m_cr);
    pango_layout_set_font_description(l_layout, (*i_page_number).m_font); 
    cairo_translate((*i_ctx).m_cr, (*i_page_number).m_pos_x, (*i_page_number).m_pos_y);

    item_page_number_get_text(l_text, i_page_number, i_ctx);

    pango_layout_set_text(l_layout, l_text, -1);

    pango_cairo_update_layout((*i_ctx).m_cr, l_layout);
    pango_cairo_show_layout((*i_ctx).m_cr, l_layout);

    cairo_restore((*i_ctx).m_cr);

    if ((*i_page_number).m_border.m_active)
    {

      item_common_cairo_dash_set((*i_ctx).m_cr, (*i_page_number).m_border.m_style);
      cairo_set_line_width((*i_ctx).m_cr, (*i_page_number).m_border.m_thickness);
      cairo_set_source_rgba(
        (*i_ctx).m_cr,
        (*i_page_number).m_border.m_color.red,
        (*i_page_number).m_border.m_color.green,
        (*i_page_number).m_border.m_color.blue,
        (*i_page_number).m_border.m_color.alpha);

      cairo_rectangle(
        (*i_ctx).m_cr,
        (*i_page_number).m_pos_x,
        (*i_page_number).m_pos_y,
        (*i_page_number).m_len_x,
        (*i_page_number).m_len_y);

      cairo_stroke((*i_ctx).m_cr);
    }

    if (0 == (*i_ctx).m_selected)
    {
      if ((*i_ctx).m_option.m_highlight &&
        (item_draw_design == (*i_ctx).m_option.m_type))
      {
        item_common_box_draw_hightlight(
          (*i_ctx).m_cr, 
          (struct geom_rect*)i_page_number,
          1.0);
      }
      break;
    }

    item_common_box_draw_grips((*i_ctx).m_cr, (struct geom_rect*)i_page_number);

  }while(0);

  if (l_font)
  {
    pango_font_description_free(l_font);
  }

  if (l_layout)
  {
    g_object_unref(l_layout);
  }

  return;
}

extern void
item_page_number_get_bbox(
  struct geom_rect*const                o_bbox,
  void const*const                      i_object)
{
  (*o_bbox)= *(struct geom_rect*)i_object;
  return;
}

extern void
item_page_number_in_event(
  enum event_type*const                 o_type,
  struct item_page_number const*const   i_page_number,
  struct item_in_event const*const      i_ctx)
{

  item_common_box_in_event(o_type, i_ctx, (struct geom_rect*)i_page_number);

  return;
}

extern void
item_page_number_move(
  struct item_page_number*const         io_page_number,
  struct geom_point const*const         i_event)
{
  item_common_box_move((struct geom_rect*)io_page_number, i_event);
  return;
}

extern void
item_page_number_normalize(
  struct item_page_number*const         io_page_number,
  double const                          i_bound_width,
  double const                          i_bound_height)
{
  item_common_box_normalize(
    (struct geom_rect*)io_page_number, 
    i_bound_width,
    i_bound_height);
  return;
}

extern int
item_page_number_read(
  GError**                              o_error,
  struct item_page_number*const         io_page_number,
  struct xml *const                     i_xml)
{
  GError*                               l_error;
  int                                   l_exit;
  int                                   l_found;
  GList*                                l_node;
  int                                   l_rc;
  GdkRGBA                               l_rgba;
  enum element_tag_type                 l_type;
  char const*                           l_value;
  struct xml*                           l_xml;

  l_error= 0;
  l_exit= 0;

  l_node= (*i_xml).m_children;

  do
  {

    if (0 == l_node)
    {
      break;
    }

    l_xml= (*l_node).data;

    item_common_lookup(&l_found, &l_type, (*l_xml).m_tag);
    g_strstrip((*l_xml).m_string->str);
    l_value= (*l_xml).m_string->str;

    if (l_found)
    {
      switch(l_type)
      {
        case element_tag_pos_x:
          (*io_page_number).m_pos_x= xml_convert_double(l_value);
          break;
        case element_tag_pos_y:
          (*io_page_number).m_pos_y= xml_convert_double(l_value);
          break;
        case element_tag_len_x:
          (*io_page_number).m_len_x= xml_convert_double(l_value);
          break;
        case element_tag_len_y:
          (*io_page_number).m_len_y= xml_convert_double(l_value);
          break;
        case element_tag_font:
          (*io_page_number).m_font= pango_font_description_from_string(l_value);
          break;
        case element_tag_color:
          l_rc= gdk_rgba_parse(&l_rgba, l_value);
          if (l_rc)
          {
            (*io_page_number).m_color= l_rgba;
          }
          break;
        case element_tag_border:
          l_exit= item_common_read_border_attr(&l_error, &(*io_page_number).m_border, l_xml);
          break;
        case element_tag_fill:
          l_exit= item_common_read_fill(&l_error, &(*io_page_number).m_fill, l_xml);
          break;
        case element_tag_type:
          (*io_page_number).m_type= page_number_all;

          do
          {
            l_rc= strcmp("even", l_value);
            if (0 == l_rc)
            {
              (*io_page_number).m_type= page_number_even;
              break;
            }

            l_rc= strcmp("odd", l_value);
            if (0 == l_rc)
            {
              (*io_page_number).m_type= page_number_odd;
              break;
            }
          }while(0);

          break;
        case element_tag_format:
          g_strlcpy((*io_page_number).m_format, l_value, sizeof((*io_page_number).m_format));
          break;
        default:
          break;
      }
    }

    l_node= (*l_node).next;

  }while(1);

  if (l_error)
  {
    g_propagate_error(o_error, l_error);
  }

  return l_exit;
}

extern void
item_page_number_resize(
  struct item_page_number*const         io_page_number,
  struct item_resize_event const*const  i_ctx)
{

  item_common_box_resize((struct geom_rect *)io_page_number, i_ctx);

  return;
}

extern int
item_page_number_write(
  GError**                              o_error,
  FILE*const                            io_fp,
  struct item_page_number const*const   i_page_number)
{
  char                                  l_buf[G_ASCII_DTOSTR_BUF_SIZE+1];
  int                                   l_exit;
  char*                                 l_text;

  l_exit= 0;
  l_text= 0;

  fprintf(io_fp, "\t\t\t<item name=\"page_number\">\n");
  fprintf(io_fp, "\t\t\t\t<pos_x>%s</pos_x>\n", g_ascii_dtostr(l_buf, sizeof(l_buf), (*i_page_number).m_pos_x));
  fprintf(io_fp, "\t\t\t\t<pos_y>%s</pos_y>\n", g_ascii_dtostr(l_buf, sizeof(l_buf), (*i_page_number).m_pos_y));
  fprintf(io_fp, "\t\t\t\t<len_x>%s</len_x>\n", g_ascii_dtostr(l_buf, sizeof(l_buf), (*i_page_number).m_len_x));
  fprintf(io_fp, "\t\t\t\t<len_y>%s</len_y>\n", g_ascii_dtostr(l_buf, sizeof(l_buf), (*i_page_number).m_len_y));
  l_text= pango_font_description_to_string((*i_page_number).m_font);
  fprintf(io_fp, "\t\t\t\t<font>%s</font>\n", l_text);
  g_free(l_text);
  l_text= gdk_rgba_to_string(&(*i_page_number).m_color);
  fprintf(io_fp, "\t\t\t\t<color>%s</color>\n", l_text);
  g_free(l_text);
  item_common_write_border_attr(o_error, io_fp, &(*i_page_number).m_border);
  item_common_write_fill(o_error, io_fp, &(*i_page_number).m_fill);

  switch((*i_page_number).m_type)
  {
    case page_number_all:
      l_text= "all";
      break;
    case page_number_even:
      l_text= "even";
      break;
    case page_number_odd:
      l_text= "odd";
      break;
    default:
      l_text=""; /* programming error */
      break;
  }

  fprintf(io_fp, "\t\t\t\t<type>%s</type>\n", l_text);
  fprintf(io_fp, "\t\t\t\t<format>%s</format>\n", (*i_page_number).m_format);
  fprintf(io_fp, "\t\t\t</item>\n");

  return l_exit;
}
